rm(list=ls())
library(tikzDevice)
## plotting parameters
one_pic_width<-4.5*(1.15^2)
one_pic_height<-one_pic_width
one_pic_pointsize<-12
two_pic_width<-4.5*1.25
two_pic_height<-two_pic_width
two_pic_pointsize<-12
three_pic_width<-4.5
three_pic_height<-three_pic_width
three_pic_pointsize<-12

LHS<-function(kappa_in_sds, gamma_c, gamma_sigma)
{
  pnorm(-gamma_c/sqrt(gamma_sigma) * kappa_in_sds) / pnorm((1-gamma_c)/sqrt(gamma_sigma) * kappa_in_sds)
}

RHS<-function(kappa_in_sds)
{
  1/pnorm(kappa_in_sds)
}

pos_diff<-function(kappa_in_sds) function(gamma_c, gamma_sigma) RHS(kappa_in_sds)>LHS(kappa_in_sds, gamma_c, gamma_sigma)

suff_cond<-function(gamma_c) (1-gamma_c)^2

gamma_sigma_when_gamma_c_zero<-function(kappa_in_sds)
{
  (kappa_in_sds / qnorm(pnorm(kappa_in_sds)/2))^2
}

num_gamma_sigma_cutoff<-function(kappa_in_sds) function(gamma_c)
{
  f<-function(x) (LHS(kappa_in_sds, gamma_c, x)-RHS(kappa_in_sds))^2
  
  optimize(f, interval = c(0,1))$minimum
}

## numerical slope at gamma_c=0
(num_gamma_sigma_cutoff(-1)(0.01)-num_gamma_sigma_cutoff(-1)(0))/0.01

rev_hazard<-function(x) dnorm(x)/pnorm(x)

d_a_d_gamma_c_when_gamma_c_zero<-function(kappa_in_sds, gamma_sigma)
{
  sqrt(gamma_sigma) * (rev_hazard(0)/rev_hazard(kappa_in_sds/sqrt(gamma_sigma))-1) 
}

kappa_in_sds_tmp<--1
gsig_tmp<-gamma_sigma_when_gamma_c_zero(kappa_in_sds_tmp)
d_a_d_gamma_c_when_gamma_c_zero(kappa_in_sds_tmp, gsig_tmp)*2*sqrt(gsig_tmp)

d_gamma_sigma_d_gamma_c_when_gamma_c_zero<-function(kappa_in_sds)
{
  gamma_sigma<-gamma_sigma_when_gamma_c_zero(kappa_in_sds)
  2*gamma_sigma * (rev_hazard(0)/rev_hazard(kappa_in_sds/sqrt(gamma_sigma))-1) 
}



sigma_theta<-sqrt(0.011+0.035) # only for ``one'' below
cal_gamma_c<-0.275
kappa_in_sds_grid<-c(-1,-2)
n_grid<-500
smallval<-0.001
gamma_c_grid<-seq(smallval,1, length=n_grid)
gamma_sigma_grid<-seq(smallval,1, length=n_grid)

par(mfrow=c(2,1))
kappa_in_sds_tmp<-kappa_in_sds_grid[1]
contour(gamma_c_grid, gamma_sigma_grid, outer(gamma_c_grid, gamma_sigma_grid, pos_diff(kappa_in_sds_tmp)), levels=1, labels="0")
title(xlab="gamma_c", ylab="gamma_sigma")
points(gamma_c_grid, suff_cond(gamma_c_grid), t="l", lty=3)
points(cal_gamma_c,0.5, col="red", pch=16)

kappa_in_sds_tmp<-kappa_in_sds_grid[2]
contour(gamma_c_grid, gamma_sigma_grid, outer(gamma_c_grid, gamma_sigma_grid, pos_diff(kappa_in_sds_tmp)), levels=1, labels="0")
title(xlab="gamma_c", ylab="gamma_sigma")
points(gamma_c_grid, suff_cond(gamma_c_grid), t="l", lty=3)
points(cal_gamma_c,0.5, col="red", pch=16)
abline(h=gamma_sigma_when_gamma_c_zero(kappa_in_sds_tmp))
par(mfrow=c(1,1))

pdf("./pdfs/prop_2_hat_gamma_sigma.pdf")
kappa_in_sds_tmp<-kappa_in_sds_grid[1]
contour(gamma_c_grid, gamma_sigma_grid, outer(gamma_c_grid, gamma_sigma_grid, pos_diff(kappa_in_sds_tmp)), levels=1, drawlabels = F, lty=1)
title(xlab=expression(gamma[c]), ylab=expression(gamma[sigma]))
points(gamma_c_grid, suff_cond(gamma_c_grid), t="l", lty=3)
#points(cal_gamma_c,0.5, col="red", pch=16)
points(c(0.275,0.4),c(0.5,.5), col="red", t="l", lwd=5)
kappa_in_sds_tmp<-kappa_in_sds_grid[2]
contour(gamma_c_grid, gamma_sigma_grid, outer(gamma_c_grid, gamma_sigma_grid, pos_diff(kappa_in_sds_tmp)), levels=1, drawlabels=F, add=T, lty=4)
legend("topright", legend=c(expression((1-gamma[c])^2), expression(hat(gamma)[sigma](kappa[sigma]==-1)),expression(hat(gamma)[sigma](kappa[sigma]==-2)), "typical values"), lty=c(3,1,4,1), bty="n", lwd=c(1,1,1,4), col=c(rep("black", 3), "red"))
dev.off()

tikz("./pdfs/prop_2_hat_gamma_sigma.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
kappa_in_sds_tmp<-kappa_in_sds_grid[1]
contour(gamma_c_grid, gamma_sigma_grid, outer(gamma_c_grid, gamma_sigma_grid, pos_diff(kappa_in_sds_tmp)), levels=1, drawlabels = F, lty=1)
title(xlab="$\\gamma_c$", ylab="$\\gamma_{\\sigma}$")
points(gamma_c_grid, suff_cond(gamma_c_grid), t="l", lty=3)
#points(cal_gamma_c,0.5, col="red", pch=16)
points(c(0.275,0.4),c(0.5,.5), col="red", t="l", lwd=5)
kappa_in_sds_tmp<-kappa_in_sds_grid[2]
contour(gamma_c_grid, gamma_sigma_grid, outer(gamma_c_grid, gamma_sigma_grid, pos_diff(kappa_in_sds_tmp)), levels=1, drawlabels=F, add=T, lty=4)
legend("topright", legend=c("$(1-\\gamma_c)^2$", "$\\hat{\\gamma}_{\\sigma}(\\kappa_{\\sigma}=-1)$","$\\hat{\\gamma}_{\\sigma}(\\kappa_{\\sigma}=-2)$", "typical $\\gamma_c,\\gamma_{\\sigma}$"), lty=c(3,1,4,1), bty="n", lwd=c(1,1,1,4), col=c(rep("black", 3), "red"), cex=1, inset=c(.03,0))
dev.off()
