library(foreign)
library(plyr)
library(xtable)
library(stargazer)
library(ggplot2)
library(texreg)
library(tikzDevice)
rm(list=ls())

###for plotting
par_mgp<-c(2.5,1,0)
cex_lab_size<-1
one_pic_width<-4.5*(1.15^2)
one_pic_height<-one_pic_width
one_pic_pointsize<-12
two_pic_width<-4.5*1.25
two_pic_height<-two_pic_width
two_pic_pointsize<-12
three_pic_width<-4.5
three_pic_height<-three_pic_width
three_pic_pointsize<-12
###

sig_level<-0.05

data<-read.dta("teachers_for_r.dta")

varname<-data$varname[1]

# 1. number of students per teacher quadratic for both math and reading!
summary(lm(count ~ tcher4 + I(tcher4^2), data=data))
summary(lm(tch_year_relative_count ~ tcher3 + I(tcher3^2), data=data))

summary(lm(count ~ tchem4 + I(tchem4^2), data=data))
summary(lm(tch_year_relative_count ~ tchem4 + I(tchem4^2), data=data))

# 2. school means and sd of teacher quality, m4
schmeans_tchem4<-with(data, tapply(tchem4, cds_code, mean))
schsds_tchem4<-with(data, tapply(tchem4, cds_code, sd))

tcher4_count_reg<-lm(count ~ tcher4 + I(tcher4^2), data=data)
tchem4_count_reg<-lm(count ~ tchem4 + I(tchem4^2), data=data)

subsetted_tcher4_count_reg<-lm(count ~ tcher4 + I(tcher4^2), data=subset(data, abs(tcher4)<=2*sd(tcher4)))

subsetted_tchem4_count_reg<-lm(count ~ tchem4 + I(tchem4^2), data=subset(data, abs(tchem4)<=2*sd(tchem4)))

quad_fit_tcher4<-function(x) sum(tcher4_count_reg$coefficients * c(1,x,x^2))
quad_fit_tchem4<-function(x) sum(tchem4_count_reg$coefficients * c(1,x,x^2))

count_plot_lim<-c(quantile(data$count, 0.25),quantile(data$count, 0.75))

tcher4_count_plot<-ggplot(data=data, aes(x=tcher4, y=count)) + geom_point(alpha=0.01) + geom_smooth(size=1) + xlim(-3*sd(data$tcher4), 3*sd(data$tcher4)) + xlab("Teacher quality, Reading") + ylab("Class size") + coord_cartesian(ylim=count_plot_lim) + stat_function(fun=Vectorize(quad_fit_tcher4), linetype="dashed") + theme_bw()

tchem4_count_plot<-ggplot(data=data, aes(x=tchem4, y=count)) + geom_point(alpha=0.01) + geom_smooth(size=1) + xlim(-3*sd(data$tchem4), 3*sd(data$tchem4)) + xlab("Teacher quality, Math") + ylab("Class size")+ coord_cartesian(ylim=count_plot_lim) + stat_function(fun=Vectorize(quad_fit_tchem4), linetype="dashed") + theme_bw()

# Fig 3a
pdf("../code_Rev1/pdfs/tcher4_count_plot.pdf")
print(tcher4_count_plot)
dev.off()

tikz("../code_Rev1/pdfs/tcher4_count_plot.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
print(ggplot(data=data, aes(x=tcher4, y=count)) + geom_smooth(size=1) + xlim(-3*sd(data$tcher4), 3*sd(data$tcher4)) + xlab("Teacher quality, Reading") + ylab("Class size") + coord_cartesian(ylim=count_plot_lim) + stat_function(fun=Vectorize(quad_fit_tcher4), linetype="dashed") + theme_bw())
dev.off()

# Fig 3b
pdf("../code_Rev1/pdfs/tchem4_count_plot.pdf")
print(tchem4_count_plot)
dev.off()

tikz("../code_Rev1/pdfs/tchem4_count_plot.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
print(ggplot(data=data, aes(x=tchem4, y=count)) + geom_smooth(size=1) + xlim(-3*sd(data$tchem4), 3*sd(data$tchem4)) + xlab("Teacher quality, Math") + ylab("Class size")+ coord_cartesian(ylim=count_plot_lim) + stat_function(fun=Vectorize(quad_fit_tchem4), linetype="dashed") + theme_bw())
dev.off()

star_0<-stargazer(tcher4_count_reg, subsetted_tcher4_count_reg, tchem4_count_reg, subsetted_tchem4_count_reg, title="Regressions of class size on teacher quality", covariate.labels=c("tcher4", "tcher4 sq", "tchem4", "tchem4 sq"), dep.var.labels="Class size", label="tab:theta_count_regs", notes.align="l")

## Table 1
write(star_0, file="./tables/tch_count_regs.tex")
