one_pic_width<-4.5*(1.15^2)
one_pic_height<-one_pic_width
one_pic_pointsize<-12
two_pic_width<-4.5*1.25
two_pic_height<-two_pic_width
two_pic_pointsize<-12
three_pic_width<-4.5
three_pic_height<-three_pic_width
three_pic_pointsize<-12

par(mfrow = c(1,1))  

#### expected output in Model HT2
sim_constant_HT2<-F
sim_variable_HT2<-F

### define class size functions at top
	## constant n
  n_const<-function(theta) 25 + theta*0

	## increasing n
	n_nonvec_inc<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 25+5*theta)) 
	n_inc<-function(theta) Vectorize(n_nonvec_inc)(theta)

	## decreasing n
	n_nonvec_dec<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 25-5*theta)) 
	n_dec<-function(theta) Vectorize(n_nonvec_dec)(theta)

  ## negative quadratic class size
  n_nonvec_neg_quad<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 25.46 - 10*theta^2)) 
  n_neg_quad<-function(theta) Vectorize(n_nonvec_neg_quad)(theta)

  ## positive quadratic class size
  n_nonvec_pos_quad<-function(theta) min(biggest_class_for_madeup_n, max(smallest_class_for_madeup_n, 24.54 + 10*theta^2)) 
  n_pos_quad<-function(theta) Vectorize(n_nonvec_pos_quad)(theta)
  
  ## increasing and then constant at zero n
  n_nonvec_piecewise<-function(theta) max(smallest_class_for_madeup_n, min(25.849,25.849+9.9225*theta))
  n_piecewise<-function(theta) Vectorize(n_nonvec_piecewise)(theta)

	## LA Times calibrated reading class size
	n_nonvec_LA_reading_cal<-function(theta) max(11, 22.702 + 1.031 * theta + -13.929 *theta^2) # 11 is smallest class size in RAND study
	
	n_LA_reading_cal<-function(theta) Vectorize(n_nonvec_LA_reading_cal)(theta)

mean_n_const<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_const(x), lower=-Inf, upper=Inf)$value
mean_n_inc<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_inc(x), lower=-Inf, upper=Inf)$value
mean_n_dec<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_dec(x), lower=-Inf, upper=Inf)$value
mean_n_neg_quad<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_neg_quad(x), lower=-Inf, upper=Inf)$value
mean_n_pos_quad<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_pos_quad(x), lower=-Inf, upper=Inf)$value
mean_n_piecewise<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_piecewise(x), lower=-Inf, upper=Inf)$value
mean_n_LA_reading_cal<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_LA_reading_cal(x), lower=-Inf, upper=Inf)$value

mean_n_by_scenario<-c(const_mean=mean_n_const, inc_mean=mean_n_inc, dec_mean=mean_n_dec, neg_quad_mean=mean_n_neg_quad, pos_quad_mean=mean_n_pos_quad, piecewise_mean=mean_n_piecewise, LA_reading_mean=mean_n_LA_reading_cal)
print(mean_n_by_scenario)

## constant class sizes
set.seed(154)
n_tmp<-1e4
constant_class_size_tmp<-25
sigma_bar_eps<-sigma_eps/sqrt(constant_class_size_tmp)
bar_eps_tmp<-rnorm(n_tmp, mean=0, sd=sigma_bar_eps)
theta_tmp<-rnorm(n_tmp, mean=0, sd=sigma_theta)
hat_theta_tmp<-theta_tmp + bar_eps_tmp

true_sd_hat_theta_tmp<-sqrt(sigma_hat_theta_sq("FE")(constant_class_size_tmp)) #sqrt(sigma_theta^2+sigma_bar_eps^2)
true_cor_hat_theta_theta<-sigma_theta/true_sd_hat_theta_tmp
constant_reliability<- (sigma_theta^2)/(true_sd_hat_theta_tmp^2)
hat_theta_EB_tmp<-hat_theta_tmp*constant_reliability
sd_hat_theta_EB_tmp<-sd(hat_theta_EB_tmp)
true_sd_hat_theta_EB_tmp<-sqrt(sigma_hat_theta_sq("EB")(constant_class_size_tmp)) #constant_reliability*true_sd_hat_theta_tmp

## HT2 objective under constant class size for FE estimator
u_HT2_FE_n_const<-function(chi_in) function(x) -chi_in*pnorm(x/true_sd_hat_theta_tmp) + (1-pnorm(x/true_sd_hat_theta_tmp))*(sigma_theta^2/true_sd_hat_theta_tmp)*(dnorm(-x/true_sd_hat_theta_tmp)/pnorm(-x/true_sd_hat_theta_tmp))

## HT2 objective under constant class size for EB estimator
u_HT2_EB_n_const<-function(chi_in) function(x) -chi_in*pnorm(x/true_sd_hat_theta_EB_tmp) + (1-pnorm(x/true_sd_hat_theta_EB_tmp))*(sigma_theta^2/true_sd_hat_theta_EB_tmp)*constant_reliability*(dnorm(-x/true_sd_hat_theta_EB_tmp)/pnorm(-x/true_sd_hat_theta_EB_tmp))

chi_tmp<- 0.1

solve_for_q_star_FE_n_const<-function(chi_in)
{
  f_tmp<-function(q_in) 
  {
    -(u_HT2_FE_n_const(chi_in)(q_in))
  }
  
  optimise(Vectorize(f_tmp), lower = -3, upper=3)
}

solve_for_q_star_EB_n_const<-function(chi_in)
{
  f_tmp<-function(q_in) 
  {
    -(u_HT2_EB_n_const(chi_in)(q_in))
  }
  
  optimise(Vectorize(f_tmp), lower = -3, upper=3)
}

value_FE_n_const<-function(chi_in) u_HT2_FE_n_const(chi_in)(solve_for_q_star_FE_n_const(chi_in)$min)
value_EB_n_const<-function(chi_in) u_HT2_EB_n_const(chi_in)(solve_for_q_star_EB_n_const(chi_in)$min)

## check FE objective
plot(u_HT2_FE_n_const(chi_tmp), xlim=c(-1,1))
abline(v=solve_for_q_star_FE_n_const(chi_tmp)$min)
abline(v= -chi_tmp/constant_reliability, col="red")
abline(h=u_HT2_FE_n_const(chi_tmp)(solve_for_q_star_FE_n_const(chi_tmp)$min))

## check EB objective
plot(u_HT2_EB_n_const(chi_tmp), xlim=c(-1,1))
abline(v=solve_for_q_star_EB_n_const(chi_tmp)$min)
abline(v= -chi_tmp, col="red")
abline(h=u_HT2_FE_n_const(chi_tmp)(solve_for_q_star_FE_n_const(chi_tmp)$min))
abline(h=u_HT2_EB_n_const(chi_tmp)(solve_for_q_star_EB_n_const(chi_tmp)$min))

chi_vec<-seq(0,0.1, length=40)
plot(chi_vec, Vectorize(value_FE_n_const)(chi_vec), t="l")
points(chi_vec, Vectorize(value_EB_n_const)(chi_vec), t="l", col="red")

####HT2 allowing for n(theta)
############## for either FE or EB
  ## E[theta|hat_theta>q]
  ## inside the integral; need to premultiply by Pr(\hat{\theta}\geq q_in
to_int_E_theta_given_hat_theta_gt_q_either<-function(estimator) function(q_in) function(theta_in)
	{
	theta_in * (1-pnorm((q_in/lambda(estimator)(n(theta_in))-theta_in), mean=0, sd=sigma_eps/sqrt(n(theta_in)))) * dnorm(theta_in, mean=mu_theta, sd=sigma_theta)
	}
## outside the integral, Pr(\hat{\theta}\geq q_in
prob_hat_theta_gt_q_either<-function(estimator) function(q_in)
	{
	f<-function(theta_in) (1-pnorm(q_in/lambda(estimator)(n(theta_in))-theta_in, mean=0, sd=sigma_eps/sqrt(n(theta_in)))) * dnorm(theta_in, mean=mu_theta, sd=sigma_theta)
	integrate(f, lower=-Inf, upper=Inf)$value
	}
	## Pr(\hat{\theta}< q_in
prob_hat_theta_lt_q_either<-function(estimator) function(q_in)
	{
	f<-function(theta_in) pnorm(q_in/lambda(estimator)(n(theta_in))-theta_in, mean=0, sd=sigma_eps/sqrt(n(theta_in))) * dnorm(theta_in, mean=mu_theta, sd=sigma_theta)
	integrate(f, lower=-Inf, upper=Inf)$value
	}

## note: this is strictly for testing. that the prob of being retained cancels out in the below function HT2_above, which is the one used for solving the admin's problem
E_theta_given_hat_theta_gt_q_either<-function(estimator) function(q_in)
	{
	1/prob_hat_theta_gt_q_either(estimator)(q_in) * integrate(function(theta_in) to_int_E_theta_given_hat_theta_gt_q_either(estimator)(q_in)(theta_in), lower=-Inf, upper=Inf)$value
	}

HT2_below<-function(estimator) function(chi_in) function(q_in)
	{
	-chi_in * prob_hat_theta_lt_q_either(estimator)(q_in)
	}

HT2_above<-function(estimator) function(chi_in) function(q_in)
	{
	integrate(function(theta_in) to_int_E_theta_given_hat_theta_gt_q_either(estimator)(q_in)(theta_in), lower=-Inf, upper=Inf)$value
	}

HT2_total<-function(estimator) function(chi_in) function(q_in)
 {
   HT2_below(estimator)(chi_in)(q_in) + HT2_above(estimator)(chi_in)(q_in)  
 }
 
q_star_lb<- -5*true_sd_hat_theta_tmp
q_star_ub<-  5*true_sd_hat_theta_tmp

solve_for_q<-function(estimator) function(chi_in)
 {
   optimize(function(q_endog) -Vectorize(HT2_total(estimator)(chi_in))(q_endog), lower=q_star_lb, upper=q_star_ub, tol=sqrt(.Machine$double.eps))
}

check_solve_for_q<-function(estimator) function(chi_in)
{
  LHS<-function(q_in) 
    {
    integrate(function(theta_in) (-chi_in/(lambda(estimator)(n(theta_in)) * sigma_eps/sqrt(n(theta_in)))) * dnorm(q_in/lambda(estimator)(n(theta_in))-theta_in, mean=0, sd=sigma_eps/sqrt(n(theta_in))) * dnorm(theta_in, mean=mu_theta, sd=sigma_theta), lower = -Inf, upper=Inf)$value
  }
  
  RHS<-function(q_in) 
    {
    integrate(function(theta_in) (theta_in/(lambda(estimator)(n(theta_in)) * sigma_eps/sqrt(n(theta_in)))) * dnorm(q_in/lambda(estimator)(n(theta_in))-theta_in, mean=0, sd=sigma_eps/sqrt(n(theta_in))) * dnorm(theta_in, mean=mu_theta, sd=sigma_theta), lower = -Inf, upper=Inf)$value
  }
  
  optimize(function(q_in) (Vectorize(LHS)(q_in) - Vectorize(RHS)(q_in))^2, lower=solve_for_q(estimator)(chi_in)$minimum*1.5, upper=q_star_ub/5, tol=sqrt(.Machine$double.eps))
}

value_n_variable<-function(estimator) function(chi_in) -solve_for_q(estimator)(chi_in)$obj
optimal_output_n_variable<-function(estimator) function(chi_in) HT2_above(estimator)(chi_in)(solve_for_q(estimator)(chi_in)$min)
optimal_cost_n_variable<-function(estimator) function(chi_in) -HT2_below(estimator)(chi_in)(solve_for_q(estimator)(chi_in)$min)

#### define:chi_vec and make values, q_star, etc. for each class size scenario
chi_vec<- seq(1/500, .3, length=30)

  # constant n
	n<-function(theta) n_const(theta)
	df_HT2_const_FE<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("FE"))(chi_vec)[1,]), value=Vectorize(value_n_variable("FE"))(chi_vec), output=Vectorize(optimal_output_n_variable("FE"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("FE"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("FE"))(chi_vec)/chi_vec)
	df_HT2_const_FE$output_per_retained<- df_HT2_const_FE$output/(1-df_HT2_const_FE$meas_replaced)
	
	df_HT2_const_EB<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("EB"))(chi_vec)[1,]), value=Vectorize(value_n_variable("EB"))(chi_vec), output=Vectorize(optimal_output_n_variable("EB"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("EB"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("EB"))(chi_vec)/chi_vec)
	df_HT2_const_EB$output_per_retained<- df_HT2_const_EB$output/(1-df_HT2_const_EB$meas_replaced)

	 if(min(abs(df_HT2_const_EB$q_star-q_star_lb))<.01 | min(abs(df_HT2_const_EB$q_star-q_star_ub))<.01) stop("q_star at boundary, const, EB")
	 if(min(abs(df_HT2_const_FE$q_star-q_star_lb))<.01 | min(abs(df_HT2_const_FE$q_star-q_star_ub))<.01) stop("q_star at boundary, const, FE")
  
   # ### Increasing n
	 n<-function(theta) n_inc(theta)
	 df_HT2_inc_FE<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("FE"))(chi_vec)[1,]), value=Vectorize(value_n_variable("FE"))(chi_vec), output=Vectorize(optimal_output_n_variable("FE"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("FE"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("FE"))(chi_vec)/chi_vec)
	 df_HT2_inc_FE$output_per_retained<- df_HT2_inc_FE$output/(1-df_HT2_inc_FE$meas_replaced)
	 
	 df_HT2_inc_EB<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("EB"))(chi_vec)[1,]), value=Vectorize(value_n_variable("EB"))(chi_vec), output=Vectorize(optimal_output_n_variable("EB"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("EB"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("EB"))(chi_vec)/chi_vec)
	 df_HT2_inc_EB$output_per_retained<- df_HT2_inc_EB$output/(1-df_HT2_inc_EB$meas_replaced)    
	  if(min(abs(df_HT2_inc_EB$q_star-q_star_lb))<.01 | min(abs(df_HT2_inc_EB$q_star-q_star_ub))<.01) stop("q_star at boundary, inc, EB") 
	  if(min(abs(df_HT2_inc_FE$q_star-q_star_lb))<.01 | min(abs(df_HT2_inc_FE$q_star-q_star_ub))<.01) stop("q_star at boundary, inc, FE") 

	 ## homo
	 df_HT2_inc_FE_homo<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec)[1,]), value=Vectorize(value_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), output=Vectorize(optimal_output_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), cost=Vectorize(optimal_cost_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec)/chi_vec)
	 df_HT2_inc_FE_homo$output_per_retained<- df_HT2_inc_FE_homo$output/(1-df_HT2_inc_FE_homo$meas_replaced)
	 
	 df_HT2_inc_EB_homo<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec)[1,]), value=Vectorize(value_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), output=Vectorize(optimal_output_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), cost=Vectorize(optimal_cost_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec)/chi_vec)
	 df_HT2_inc_EB_homo$output_per_retained<- df_HT2_inc_EB_homo$output/(1-df_HT2_inc_EB_homo$meas_replaced)
	 
	  if(min(abs(df_HT2_inc_EB_homo$q_star-q_star_lb))<.01 | min(abs(df_HT2_inc_EB_homo$q_star-q_star_ub))<.01) stop("q_star at boundary, inc, EB homo")
	  if(min(abs(df_HT2_inc_FE_homo$q_star-q_star_lb))<.01 | min(abs(df_HT2_inc_FE_homo$q_star-q_star_ub))<.01) stop("q_star at boundary, inc, FE homo")
	 
   # ### Decreasing n
	 n<-function(theta) n_dec(theta)
	 df_HT2_dec_FE<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("FE"))(chi_vec)[1,]), value=Vectorize(value_n_variable("FE"))(chi_vec), output=Vectorize(optimal_output_n_variable("FE"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("FE"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("FE"))(chi_vec)/chi_vec)
	 df_HT2_dec_FE$output_per_retained<- df_HT2_dec_FE$output/(1-df_HT2_dec_FE$meas_replaced)
	 
	 df_HT2_dec_EB<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("EB"))(chi_vec)[1,]), value=Vectorize(value_n_variable("EB"))(chi_vec), output=Vectorize(optimal_output_n_variable("EB"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("EB"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("EB"))(chi_vec)/chi_vec)
	 df_HT2_dec_EB$output_per_retained<- df_HT2_dec_EB$output/(1-df_HT2_dec_EB$meas_replaced)    
	  if(min(abs(df_HT2_dec_EB$q_star-q_star_lb))<.01 | min(abs(df_HT2_dec_EB$q_star-q_star_ub))<.01) stop("q_star at boundary, dec, EB") 
	  if(min(abs(df_HT2_dec_FE$q_star-q_star_lb))<.01 | min(abs(df_HT2_dec_FE$q_star-q_star_ub))<.01) stop("q_star at boundary, dec, FE") 
  
   ## negative quadratic class size
	n<-function(theta) n_neg_quad(theta)
	df_HT2_neg_quad_FE<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("FE"))(chi_vec)[1,]), value=Vectorize(value_n_variable("FE"))(chi_vec), output=Vectorize(optimal_output_n_variable("FE"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("FE"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("FE"))(chi_vec)/chi_vec)
	df_HT2_neg_quad_FE$output_per_retained<- df_HT2_neg_quad_FE$output/(1-df_HT2_neg_quad_FE$meas_replaced)
	
	df_HT2_neg_quad_EB<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("EB"))(chi_vec)[1,]), value=Vectorize(value_n_variable("EB"))(chi_vec), output=Vectorize(optimal_output_n_variable("EB"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("EB"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("EB"))(chi_vec)/chi_vec)
	df_HT2_neg_quad_EB$output_per_retained<- df_HT2_neg_quad_EB$output/(1-df_HT2_neg_quad_EB$meas_replaced)

	 if(min(abs(df_HT2_neg_quad_EB$q_star-q_star_lb))<.01 | min(abs(df_HT2_neg_quad_EB$q_star-q_star_ub))<.01) stop("q_star at boundary, neg quad, EB")
	 if(min(abs(df_HT2_neg_quad_FE$q_star-q_star_lb))<.01 | min(abs(df_HT2_neg_quad_FE$q_star-q_star_ub))<.01) stop("q_star at boundary, neg quad, FE")
  
	# homo
	df_HT2_neg_quad_FE_homo<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec)[1,]), value=Vectorize(value_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), output=Vectorize(optimal_output_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), cost=Vectorize(optimal_cost_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec)/chi_vec)
	df_HT2_neg_quad_FE_homo$output_per_retained<- df_HT2_neg_quad_FE_homo$output/(1-df_HT2_neg_quad_FE_homo$meas_replaced)
	
	df_HT2_neg_quad_EB_homo<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec)[1,]), value=Vectorize(value_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), output=Vectorize(optimal_output_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), cost=Vectorize(optimal_cost_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec)/chi_vec)
	df_HT2_neg_quad_EB_homo$output_per_retained<- df_HT2_neg_quad_EB_homo$output/(1-df_HT2_neg_quad_EB_homo$meas_replaced)
	
	 if(min(abs(df_HT2_neg_quad_EB_homo$q_star-q_star_lb))<.01 | min(abs(df_HT2_neg_quad_EB_homo$q_star-q_star_ub))<.01) stop("q_star at boundary, neg quad, EB homo") 
	 if(min(abs(df_HT2_neg_quad_FE_homo$q_star-q_star_lb))<.01 | min(abs(df_HT2_neg_quad_FE_homo$q_star-q_star_ub))<.01) stop("q_star at boundary, neg_quad, FE homo") 
	
  ## positive quadratic class size
  n<-function(theta) n_pos_quad(theta)
  df_HT2_pos_quad_FE<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("FE"))(chi_vec)[1,]), value=Vectorize(value_n_variable("FE"))(chi_vec), output=Vectorize(optimal_output_n_variable("FE"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("FE"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("FE"))(chi_vec)/chi_vec)
  df_HT2_pos_quad_FE$output_per_retained<- df_HT2_pos_quad_FE$output/(1-df_HT2_pos_quad_FE$meas_replaced)
  
  df_HT2_pos_quad_EB<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("EB"))(chi_vec)[1,]), value=Vectorize(value_n_variable("EB"))(chi_vec), output=Vectorize(optimal_output_n_variable("EB"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("EB"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("EB"))(chi_vec)/chi_vec)
  df_HT2_pos_quad_EB$output_per_retained<- df_HT2_pos_quad_EB$output/(1-df_HT2_pos_quad_EB$meas_replaced)
  
   if(min(abs(df_HT2_pos_quad_EB$q_star-q_star_lb))<.01 | min(abs(df_HT2_pos_quad_EB$q_star-q_star_ub))<.01) stop("q_star at boundary, pos quad EB")
   if(min(abs(df_HT2_pos_quad_FE$q_star-q_star_lb))<.01 | min(abs(df_HT2_pos_quad_FE$q_star-q_star_ub))<.01) stop("q_star at boundary, pos quad FE")
  
  # ### Piecewise n
  n<-function(theta) n_piecewise(theta)
  df_HT2_piecewise_FE<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("FE"))(chi_vec)[1,]), value=Vectorize(value_n_variable("FE"))(chi_vec), output=Vectorize(optimal_output_n_variable("FE"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("FE"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("FE"))(chi_vec)/chi_vec)
  df_HT2_piecewise_FE$output_per_retained<- df_HT2_piecewise_FE$output/(1-df_HT2_piecewise_FE$meas_replaced)
  
  df_HT2_piecewise_EB<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q("EB"))(chi_vec)[1,]), value=Vectorize(value_n_variable("EB"))(chi_vec), output=Vectorize(optimal_output_n_variable("EB"))(chi_vec), cost=Vectorize(optimal_cost_n_variable("EB"))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable("EB"))(chi_vec)/chi_vec)
  df_HT2_piecewise_EB$output_per_retained<- df_HT2_piecewise_EB$output/(1-df_HT2_piecewise_EB$meas_replaced)    
   if(min(abs(df_HT2_piecewise_EB$q_star-q_star_lb))<.01 | min(abs(df_HT2_piecewise_EB$q_star-q_star_ub))<.01) stop("q_star at boundary, piecewise, EB") 
   if(min(abs(df_HT2_piecewise_FE$q_star-q_star_lb))<.01 | min(abs(df_HT2_piecewise_FE$q_star-q_star_ub))<.01) stop("q_star at boundary, piecewise, FE") 
  
  ## homo
  df_HT2_piecewise_FE_homo<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec)[1,]), value=Vectorize(value_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), output=Vectorize(optimal_output_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), cost=Vectorize(optimal_cost_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable_homo("FE")(sigma_eps_bar_homo_given_n(n)))(chi_vec)/chi_vec)
  df_HT2_piecewise_FE_homo$output_per_retained<- df_HT2_piecewise_FE_homo$output/(1-df_HT2_piecewise_FE_homo$meas_replaced)
  
  df_HT2_piecewise_EB_homo<-data.frame(chi=chi_vec, q_star=unlist(Vectorize(solve_for_q_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec)[1,]), value=Vectorize(value_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), output=Vectorize(optimal_output_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), cost=Vectorize(optimal_cost_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec), meas_replaced=Vectorize(optimal_cost_n_variable_homo("EB")(sigma_eps_bar_homo_given_n(n)))(chi_vec)/chi_vec)
  df_HT2_piecewise_EB_homo$output_per_retained<- df_HT2_piecewise_EB_homo$output/(1-df_HT2_piecewise_EB_homo$meas_replaced)
  
   if(min(abs(df_HT2_piecewise_EB_homo$q_star-q_star_lb))<.01 | min(abs(df_HT2_piecewise_EB_homo$q_star-q_star_ub))<.01) stop("q_star at boundary, piecewise, EB homo") 
   if(min(abs(df_HT2_piecewise_FE_homo$q_star-q_star_lb))<.01 | min(abs(df_HT2_piecewise_FE_homo$q_star-q_star_ub))<.01) stop("q_star at boundary, piecewise, FE homo") 
  
HT2_value_plot_madeup_ylim<-c(.975,1.025)

pdf("./pdfs/HT2_value_ratio_all.pdf")
  par(mgp=par_mgp)
  ylim_tmp<-range(c(df_HT2_const_FE$value/df_HT2_const_EB$value,df_HT2_neg_quad_FE$value/df_HT2_neg_quad_EB$value,df_HT2_pos_quad_FE$value/df_HT2_pos_quad_EB$value, df_HT2_dec_FE$value/df_HT2_dec_EB$value, df_HT2_inc_FE$value/df_HT2_inc_EB$value ))
  
  # constant n
  plot(chi_vec, df_HT2_const_FE$value/df_HT2_const_EB$value, xlab=expression("Replacement cost" ~ chi), ylab=expression(v(chi)["HT2"]^{"FE"} / v(chi)["HT2"]^{"EB"}), cex.lab=cex_lab_size, t="l", axes=F, ylim=ylim_tmp, lty=3)
  #  abline(h=1, lty=3)
  
  Axis(side=1, labels=T)
  axis(side=2, at=1, tick=F)
  Axis(side=2, labels=F)
  
#   ### Increasing n
   points(chi_vec, df_HT2_inc_FE$value/df_HT2_inc_EB$value, t="l", lty=2, col="red")
#   ### Decreasing n
   points(chi_vec, df_HT2_dec_FE$value/df_HT2_dec_EB$value, t="l", lty=6, col="green")
  
   ## negative quadratic class size
   points(chi_vec, df_HT2_neg_quad_FE$value/df_HT2_neg_quad_EB$value, t="l", lty=4, col="blue")
  
  ## positive quadratic class size
  points(chi_vec, df_HT2_pos_quad_FE$value/df_HT2_pos_quad_EB$value, t="l", lty=5, col="brown")
  
  legend("topleft",c("constant", "inc.", "dec.", "neg. quad", "pos. quad"), lty = c(3,2,6,4,5), col=c("black","red","green", "blue", "brown"), bty="n")

dev.off()

tikz("./pdfs/HT2_value_ratio_all.tex", standAlone=F, width=three_pic_width, height=three_pic_height, pointsize=three_pic_pointsize)
par(mgp=par_mgp)
ylim_tmp<-range(c(df_HT2_const_FE$value/df_HT2_const_EB$value,df_HT2_neg_quad_FE$value/df_HT2_neg_quad_EB$value,df_HT2_pos_quad_FE$value/df_HT2_pos_quad_EB$value, df_HT2_dec_FE$value/df_HT2_dec_EB$value, df_HT2_inc_FE$value/df_HT2_inc_EB$value ))
# constant n
plot(chi_vec, df_HT2_const_FE$value/df_HT2_const_EB$value, xlab="Replacement cost $\\chi$", ylab="Ratio of administrator value $v_{HT2}^{FE}(\\chi)/v_{HT2}^{EB}(\\chi)$", cex.lab=cex_lab_size, t="l", axes=F, ylim=ylim_tmp, lty=3, xlim=c(0,max(chi_vec)))

Axis(side=1, labels=T)
axis(side=2, at=1, tick=F)
Axis(side=2, labels=F)

#   ### Increasing n
points(chi_vec, df_HT2_inc_FE$value/df_HT2_inc_EB$value, t="l", lty=2, col="red")
#   ### Decreasing n
points(chi_vec, df_HT2_dec_FE$value/df_HT2_dec_EB$value, t="l", lty=6, col="green")

## negative quadratic class size
points(chi_vec, df_HT2_neg_quad_FE$value/df_HT2_neg_quad_EB$value, t="l", lty=4, col="blue")

## positive quadratic class size
points(chi_vec, df_HT2_pos_quad_FE$value/df_HT2_pos_quad_EB$value, t="l", lty=5, col="brown")

legend("topleft",c("constant", "inc.", "dec.", "neg. quad", "pos. quad"), lty = c(3,2,6,4,5), col=c("black","red","green", "blue", "brown"), bty="n")

dev.off()



## calibrated effect using LA Times reading
## Section 5.3 results
n<-function(theta) n_LA_reading_cal(theta)

#calibrated_chi<-0.07 ## value from Kaine and Staiger JEP
calibrated_chi<-0.054
print(paste("calibrated chi:", calibrated_chi))
print(paste("value at EB:", value_n_variable("EB")(calibrated_chi)))
print(paste("q^FE, q^EB:", solve_for_q("FE")(calibrated_chi)$min, solve_for_q("EB")(calibrated_chi)$min))
print(paste("pctile FE, EB:", optimal_cost_n_variable("FE")(calibrated_chi)/calibrated_chi
, optimal_cost_n_variable("EB")(calibrated_chi)/calibrated_chi))
print(paste("optimal output at EB:", optimal_output_n_variable("EB")(calibrated_chi)))
pct_higher_value_under_FE<-value_n_variable("FE")(calibrated_chi)/value_n_variable("EB")(calibrated_chi)-1
pct_higher_cost_under_FE<-optimal_cost_n_variable("FE")(calibrated_chi)/optimal_cost_n_variable("EB")(calibrated_chi)-1
pct_higher_output_under_FE<-optimal_output_n_variable("FE")(calibrated_chi)/optimal_output_n_variable("EB")(calibrated_chi)-1
print(c(pct_higher_cost_under_FE, pct_higher_output_under_FE, pct_higher_value_under_FE))

## from Hanushek (2011): teacher with 1sd higher VA generates gains of $400,000 (present discounted value) with class size of 20 students.
earnings_increase_FE_over_EB<-optimal_output_n_variable("EB")(calibrated_chi)*pct_higher_output_under_FE*400000*n_LAUSD_teachers*22.5/20

calibrated_chi<-0.07
print(paste("calibrated chi:", calibrated_chi))
print(paste("value at EB:", value_n_variable("EB")(calibrated_chi)))
print(paste("q^FE, q^EB:", solve_for_q("FE")(calibrated_chi)$min, solve_for_q("EB")(calibrated_chi)$min))
print(paste("optimal output at EB:", optimal_output_n_variable("EB")(calibrated_chi)))
print(paste("pctile FE, EB:", optimal_cost_n_variable("FE")(calibrated_chi)/calibrated_chi, optimal_cost_n_variable("EB")(calibrated_chi)/calibrated_chi))
pct_higher_value_under_FE<-value_n_variable("FE")(calibrated_chi)/value_n_variable("EB")(calibrated_chi)-1
pct_higher_cost_under_FE<-optimal_cost_n_variable("FE")(calibrated_chi)/optimal_cost_n_variable("EB")(calibrated_chi)-1
pct_higher_output_under_FE<-optimal_output_n_variable("FE")(calibrated_chi)/optimal_output_n_variable("EB")(calibrated_chi)-1
print(c(pct_higher_cost_under_FE, pct_higher_output_under_FE, pct_higher_value_under_FE))

## from Hanushek (2011): teacher with 1sd higher VA generates gains of $400,000 (present discounted value) with class size of 20 students.
earnings_increase_FE_over_EB<-optimal_output_n_variable("EB")(calibrated_chi)*pct_higher_output_under_FE*400000*n_LAUSD_teachers*22.5/20

