## solve for optimal action in linear contract
data<-read.dta("../la_times_data/teachers_for_r.dta")
CARA<-0.35 ## from Table 2 in \cite{babcock1993risk}  
target_output<-0.01
w_underbar<-70000
bar_sigma_sq_eps_LAUSD<-mean(sigma_sq_eps/data$count)*1000
action<-function(bar_sigma_sq_eps_in) function(cost_in) (1/(cost_in*(1+CARA*cost_in*bar_sigma_sq_eps_in)))
beta_1<-function(bar_sigma_sq_eps_in) function(cost_in) (1/(1+CARA*cost_in*bar_sigma_sq_eps_in))
beta_0<-function(bar_sigma_sq_eps_in) function(cost_in) 
{
  w_underbar - beta_1(bar_sigma_sq_eps_in)(cost_in)^2/cost_in + (cost_in/2)*(beta_1(bar_sigma_sq_eps_in)(cost_in)^2/cost_in^2) + (CARA/2)*beta_1(bar_sigma_sq_eps_in)(cost_in)^2*bar_sigma_sq_eps_in
}

variable_pay<-function(bar_sigma_sq_eps_in) function(cost_in) max(1e-5,beta_1(bar_sigma_sq_eps_in)(cost_in)*action(bar_sigma_sq_eps_in)(cost_in))
income<-function(bar_sigma_sq_eps_in) function(cost_in) beta_0(bar_sigma_sq_eps_in)(cost_in) + variable_pay(bar_sigma_sq_eps_in)(cost_in)

to_calibrate_sigma_nu_gamma<-function(pars_in)
{
  sigma_eta_sq_in<-pars_in[1]
  cost_in<-pars_in[2]
  scaling<-abs(beta_1_target/log(variable_pay_target))*2
  (beta_1(sigma_eta_sq_in)(cost_in)-beta_1_target)^2 + (scaling*(log(variable_pay(sigma_eta_sq_in)(cost_in))-log(variable_pay_target)))^2
}

b1_target_vec<-seq(.01, .5, length=5)
vpt_vec<-exp(seq(log(50), log(20000), length=5))
CARA_vec<-exp(seq(log(1e-3), log(2), length=5))

df_HA_cal<-data.frame(beta_1_target=NA, variable_pay_target=NA, CARA=NA, sigma_eta_sq_cal=NA, cost_cal=NA, beta_1_cal=NA, variable_pay_cal=NA, action_FE_cal=NA, action_FE_over_action_EB_cal=NA, optim_error=NA)

z_mat<-array(dim=c(length(vpt_vec), length(CARA_vec), length(b1_target_vec)))
ii<-1
for(variable_pay_target in vpt_vec)
{
  jj<-1
  for(CARA in CARA_vec)
  {
    kk<-1
    for(beta_1_target in b1_target_vec)
    {
      cal_HA_pars<-optim(par=c(50000,.0001), to_calibrate_sigma_nu_gamma, control=list(parscale=c(1,1e-7)))
      sigma_eta_sq_cal<-cal_HA_pars$par[1]
      cost_cal<-cal_HA_pars$par[2]
      df_HA_cal<-rbind(df_HA_cal, c(beta_1_target, variable_pay_target, CARA, sigma_eta_sq_cal, cost_cal, beta_1(sigma_eta_sq_cal)(cost_cal), variable_pay(sigma_eta_sq_cal)(cost_cal), action(sigma_eta_sq_cal)(cost_cal), action(sigma_eta_sq_cal)(cost_cal)/action(sigma_eta_sq_cal/FE_over_EB_sig_vs_noise)(cost_cal), optim_error=cal_HA_pars$value))
      
      z_mat[ii,jj,kk]<-action(sigma_eta_sq_cal)(cost_cal)/action(sigma_eta_sq_cal/FE_over_EB_sig_vs_noise)(cost_cal)
      kk<-kk+1
    }
    jj<-jj+1
  }
  print(paste(ii, "/", length(vpt_vec)))
  ii<-ii+1
}
df_HA_cal<-df_HA_cal[-1,]

print(summary(df_HA_cal))

## from Muralidharan and Sundararaman (2011)
program_effect_in_sd<-0.283/2 # table 8, top row, col (4)
rupees_per_dollar<-45
students_per_teacher<-37.5 # table 2, column 3
program_effect_math_in_sd<-0.329 #table 8, row 1, col (5)
program_effect_telugu_in_sd<-0.239 #table 8, row 1, col (5)
rupee_gain_per_student<-28000*(program_effect_math_in_sd*0.16+program_effect_telugu_in_sd*0.20) # formula is in middle of manuscript page 72
output_per_student<-rupee_gain_per_student/rupees_per_dollar # footnote 34, page 72, but using growth rate of 0.1004, calculated using test score gains in SDs from individual incentives for math (col (5)) and telegu (col (6)) in table 8
output_per_teacher<-output_per_student*students_per_teacher
payment_per_teacher<-(20000/rupees_per_dollar)/3.14 # 3.14 teachers per incentive school (Table 2, col (3)); 10,000 is from bottom of page 70
program_slope<-payment_per_teacher/output_per_teacher
gamma_cal<-program_slope/output_per_teacher
output_increase_per_sd_test_score<-output_per_student*students_per_teacher/program_effect_in_sd

sigma_eta_sq_cal_base<-output_increase_per_sd_test_score^2*sigma_sq_eps/students_per_teacher
sigma_eta_sq_cal<-sigma_eta_sq_cal_base
CARA_base<-6.7e-3 # from Cohen and Einav (2007), Table 5, mean CARA of benchmark model
CARA<-CARA_base
beta_1(sigma_eta_sq_cal)(gamma_cal)
action(sigma_eta_sq_cal)(gamma_cal)
action(sigma_eta_sq_cal)(gamma_cal)/action(sigma_eta_sq_cal*1.032)(gamma_cal)
## action in sd of test scores
action(sigma_eta_sq_cal)(gamma_cal)/output_increase_per_sd_test_score

CARA_grid<-seq(0.5*CARA_base, 10*CARA_base, length=40)
sigma_eta_sq_cal_grid<-seq(0.5*sigma_eta_sq_cal_base, 10*sigma_eta_sq_cal_base, length=40)
b1_mat<-matrix(NA, nrow=length(CARA_grid), ncol=length(sigma_eta_sq_cal_grid)) 
action_fe_mat<-b1_mat
action_ratio_mat<-b1_mat
action_fe_in_sd_mat<-b1_mat
frac_variable_pay_fe_mat<-b1_mat
ii<-1
for(CARA in CARA_grid)
{
 jj<-1
	for(sigma_eta_sq_cal in sigma_eta_sq_cal_grid)
	{
	b1_mat[ii,jj]<-beta_1(sigma_eta_sq_cal)(gamma_cal)
	frac_variable_pay_fe_mat[ii,jj]<-variable_pay(sigma_eta_sq_cal)(gamma_cal) / income(sigma_eta_sq_cal)(gamma_cal)
	action_fe_mat[ii,jj]<-action(sigma_eta_sq_cal)(gamma_cal)
	action_fe_in_sd_mat[ii,jj]<-action(sigma_eta_sq_cal)(gamma_cal)/output_increase_per_sd_test_score
	action_ratio_mat[ii,jj]<-action(sigma_eta_sq_cal)(gamma_cal)/action(sigma_eta_sq_cal*1.032)(gamma_cal)
	jj<-jj+1
	}
	ii<-ii+1
}

pt_cex<-1.5
pdf("./pdfs/HA_robust_b1.pdf")
  par(mgp=par_mgp)
	contour(CARA_grid, log(sigma_eta_sq_cal_grid), b1_mat, xlab=expression(paste("CARA parameter ", xi)), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(CARA_base, log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_variable_pay_share.pdf")
  par(mgp=par_mgp)
	contour(CARA_grid, log(sigma_eta_sq_cal_grid), frac_variable_pay_fe_mat, xlab=expression(paste("CARA parameter ", xi)), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(CARA_base, log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_action.pdf")
  par(mgp=par_mgp)
	contour(CARA_grid, log(sigma_eta_sq_cal_grid), action_fe_mat, xlab=expression(paste("CARA parameter ", xi)), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(CARA_base, log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_action_in_sd.pdf")
  par(mgp=par_mgp)
	contour(CARA_grid, log(sigma_eta_sq_cal_grid), action_fe_in_sd_mat, xlab=expression(paste("CARA parameter ", xi)), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(CARA_base, log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_action_ratio.pdf")
  par(mgp=par_mgp)
	contour(CARA_grid, log(sigma_eta_sq_cal_grid), action_ratio_mat, xlab=expression(paste("CARA parameter ", xi)), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(CARA_base, log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_b1_both_log.pdf")
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), b1_mat, xlab=expression(paste("Log of CARA parameter ", log(xi))), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_variable_pay_share_both_log.pdf")
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), frac_variable_pay_fe_mat, xlab=expression(paste("Log of CARA parameter ", log(xi))), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_action_both_log.pdf")
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), action_fe_mat, xlab=expression(paste("Log of CARA parameter ", log(xi))), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_action_in_sd_both_log.pdf")
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), action_fe_in_sd_mat, xlab=expression(paste("Log of CARA parameter ", log(xi))), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

pdf("./pdfs/HA_robust_action_ratio_both_log.pdf")
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), action_ratio_mat, xlab=expression(paste("Log of CARA parameter ", log(xi))), ylab=expression(paste("Log of output noise ", log(sigma[eta]^2))), cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend("topright", legend=expression(paste("at calibrated values of ", sigma[eta]^2,",", xi)), pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

## tikz pics
tikz("./pdfs/HA_robust_b1_both_log.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), b1_mat, xlab="Log of CARA parameter $log(\\xi)$", ylab=
"Log of output noise $log(\\sigma_{\\eta}^2$)", cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend(-3.7,18, legend="at calibrated $\\sigma_{\\eta}^2,\\xi$", pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

tikz("./pdfs/HA_robust_variable_pay_share_both_log.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), frac_variable_pay_fe_mat, xlab="Log of CARA parameter $log(\\xi)$", ylab=
"Log of output noise $log(\\sigma_{\\eta}^2$)", cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend(-3.7,18, legend="at calibrated $\\sigma_{\\eta}^2,\\xi$", pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

tikz("./pdfs/HA_robust_action_both_log.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), action_fe_mat, xlab="Log of CARA parameter $log(\\xi)$", ylab=
"Log of output noise $log(\\sigma_{\\eta}^2$)", cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend(-3.7,18, legend="at calibrated $\\sigma_{\\eta}^2,\\xi$", pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

tikz("./pdfs/HA_robust_action_in_sd_both_log.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), action_fe_in_sd_mat, xlab="Log of CARA parameter $log(\\xi)$", ylab=
"Log of output noise $log(\\sigma_{\\eta}^2$)", cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend(-3.7,18, legend="at calibrated $\\sigma_{\\eta}^2,\\xi$", pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()

tikz("./pdfs/HA_robust_action_ratio_both_log.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
  par(mgp=par_mgp)
	contour(log(CARA_grid), log(sigma_eta_sq_cal_grid), action_ratio_mat, xlab="Log of CARA parameter $log(\\xi)$", ylab=
"Log of output noise $log(\\sigma_{\\eta}^2$)", cex.lab=cex_lab_size, labcex=pt_cex/1.5)
	points(log(CARA_base), log(sigma_eta_sq_cal_base), cex=pt_cex, col="red", pch=16)
	legend(-3.7,18, legend="at calibrated $\\sigma_{\\eta}^2,\\xi$", pch=16, bty="n", pt.cex=pt_cex, col="red")
dev.off()
