rm(list=ls())
library(tikzDevice)
## plotting parameters
one_pic_width<-4.5*(1.15^2)
one_pic_height<-one_pic_width
one_pic_pointsize<-12
two_pic_width<-4.5*1.25
two_pic_height<-two_pic_width
two_pic_pointsize<-12
three_pic_width<-4.5
three_pic_height<-three_pic_width
three_pic_pointsize<-12

chi<-0.10
q<--chi
sigma_theta<-0.25
sigma_bar_eps<-0.25
gamma_sigma<-sigma_theta^2/(sigma_bar_eps^2 + sigma_theta^2)
mu_P<-q
sigma_P<-sqrt(sigma_bar_eps^2 * gamma_sigma)

theta_vec<-seq(-2*sigma_theta,0*sigma_theta, length=500)

m1<-function(theta) q*theta*(chi+theta)

m2<-function(theta) m1(theta)*dnorm(q-theta, mean=0, sd=sigma_theta)

m_3<-function(theta) m2(theta)*dnorm(theta, mean=0, sd=sigma_theta)
plot(theta_vec, m_3(theta_vec), t="l")
abline(h=0, lty=3)

f_P_div_S_P<-function(theta) dnorm(theta, mean=mu_P, sd=sigma_P)

print(f_P_div_S_P(-chi))
print("should be negative...")
print(c(chi, integrate(m_3, lower = -Inf, upper=0)$value))

x_vec<-seq(0,3*sigma_theta, length=200)
## from Shah (1985)
pnorm_poly_approx<-function(z) 
  {
  if(z<=2.2)
    {
    1/2 + z*(4.4-z)/10
  } else {
    if(z<=2.6) 
      {
      1/2 + 0.49
    } else {
      1
    }
  }
}

## calculate error for pnorm approximation a la Shah
pnorm_error<-function(x) pnorm(x) - Vectorize(pnorm_poly_approx)(x)
xtmp<-seq(0,5,length=5000)
plot(xtmp, pnorm(xtmp), t="l", xlab="", ylab="")
points(xtmp, Vectorize(pnorm_poly_approx)(xtmp), t="l", col="red", lty=3)
plot(xtmp,pnorm_error(xtmp), t="l")
integrate(pnorm_error, lower=0, upper=Inf)
summary(pnorm_error(xtmp))

pnorm_poly_approx_nonstandard<-function(mean_in,sd_in) function(x) 
  {
  z<-(x-mean_in)/sd_in
  if(z<=2.2)
  {
    1/2 + z*(4.4-z)/10
  } else {
    if(z<=2.6) 
    {
      1/2 + 0.49
    } else {
      1
    }
  }
  
}

plot(x_vec, pnorm(x_vec, mean=-1,sd=1.5), t="l")
points(x_vec, Vectorize(pnorm_poly_approx_nonstandard(mean_in=-1,sd_in = 1.5))(x_vec), t="l", lty=3, col="red")

dnorm_poly_approx<-function(z) (4.4-2*z)/10
dnorm_poly_approx_nonstandard<-function(mean_in,sd_in) function(x) 
{
  #z<-(x-mean_in)/sd_in
  #(4.4/sd_in-2*z/sd_in)/10
  z<-(x-mean_in)/sd_in
  if(abs(z)<2.2)
  {
    (4.4/sd_in-2*abs(x-mean_in)/sd_in^2)/10
  } else {
    if(abs(z)==2.2) 
    {
      0.1
    } else {
      0
    }
  }
}
mu_tmp<-0.0
sd_tmp<-sqrt((sigma_theta^2)/2)
plot(-x_vec, dnorm(-x_vec, mean=mu_tmp,sd=sd_tmp), t="l")
points(-x_vec, Vectorize(dnorm_poly_approx_nonstandard(mean_in=mu_tmp,sd_in=sd_tmp))(-x_vec), t="l", lty=3, col="red")
points(-(2.2-mu_tmp)/sd_tmp, Vectorize(dnorm_poly_approx_nonstandard(mean_in=mu_tmp,sd_in=sd_tmp))((2.2-mu_tmp)/sd_tmp), col="red")
abline(a=2.45,b=1/(5*sd_tmp^2))

tikz("./pdfs/prop_9_m_theta.tex", standAlone=F, width=three_pic_width, height=three_pic_height, pointsize=three_pic_pointsize)
par(mfrow=c(3,1))
plot(theta_vec, abs(m1(theta_vec)), t="l", xlab="Teacher quality $\\theta$", ylab="$|m(\\theta)|$", ylim=c(0,0.005))
abline(h=0, lty=3)
abline(v=-chi, lty=3)

plot(theta_vec, Vectorize(dnorm_poly_approx_nonstandard(mean_in=mu_P,sd_in=sigma_P))(theta_vec), t="l", lty=3, col="red", xlab="Teacher quality $\\theta$", ylab="density")
points(theta_vec, f_P_div_S_P(theta_vec), t="l")
legend("topleft", legend=c("actual, $f_P$", "approx, $\\tilde{f}_P$"), lty=c(1,3), col=c("black", "red"), bty="n")
abline(v=-chi, lty=3)

plot(theta_vec, abs(m1(theta_vec))*Vectorize(dnorm_poly_approx_nonstandard(mean_in=mu_P,sd_in=sigma_P))(theta_vec), , t="l", lty=3, col="red", xlab="Teacher quality $\\theta$", ylab="$|m(\\theta)| f_P(\\theta)$")
points(theta_vec, abs(m1(theta_vec))*f_P_div_S_P(theta_vec), t="l")
abline(h=0, lty=3)
abline(v=-chi, lty=3)
legend("topright", legend=c("actual", "approx"), lty=c(1,3), col=c("black", "red"), bty="n")

par(mfrow=c(1,1))
dev.off()

pdf("./pdfs/prop_9_m_theta.pdf")
par(mfrow=c(3,1))
plot(theta_vec, abs(m1(theta_vec)), t="l", xlab=expression(theta), ylab=expression(abs(m(theta))), ylim=c(0,0.005))
abline(h=0, lty=3)
abline(v=-chi, lty=3)
plot(theta_vec, Vectorize(dnorm_poly_approx_nonstandard(mean_in=mu_P,sd_in=sigma_P))(theta_vec), t="l", lty=3, col="red", xlab=expression(theta), ylab="density")
points(theta_vec, f_P_div_S_P(theta_vec), t="l")
legend("topleft", legend=c("actual", "approx"), lty=c(1,3), col=c("black", "red"), bty="n")
abline(v=-chi, lty=3)
plot(theta_vec, abs(m1(theta_vec))*Vectorize(dnorm_poly_approx_nonstandard(mean_in=mu_P,sd_in=sigma_P))(theta_vec), , t="l", lty=3, col="red", xlab=expression(theta), ylab=expression(abs(m(theta)) * f[P]))
points(theta_vec, abs(m1(theta_vec))*f_P_div_S_P(theta_vec), t="l")
abline(h=0, lty=3)
abline(v=-chi, lty=3)
legend("topright", legend=c("actual", "approx"), lty=c(1,3), col=c("black", "red"), bty="n")
par(mfrow=c(1,1))
dev.off()

tikz("./pdfs/prop_9_m_theta_f_P.tex", standAlone=F, width=three_pic_width, height=three_pic_height, pointsize=three_pic_pointsize)
par(mfrow=c(1,1))
par(mar = c(5, 4, 4, 4) + 0.3)
plot(theta_vec, abs(m1(theta_vec)), t="l", xlab="Teacher quality $\\theta$", ylab="$|m(\\theta)|$", ylim=c(0,0.005))
abline(h=0, lty=3)
abline(v=-chi, lty=2)
par(new = TRUE)
plot(theta_vec, f_P_div_S_P(theta_vec), t="l", axes=F, xlab="", ylab="", lty=4)
axis(side=4, at = pretty(range(f_P_div_S_P(theta_vec)))[-c(1,6)], labels=F)
#mtext("$f_P(\\theta)$", side=4, line=3)
corners = par("usr") #Gets the four corners of plot area (x1, x2, y1, y2)
par(xpd = TRUE) #Draw outside plot area
text(x = corners[2]+.075, y = mean(corners[3:4]), "$f_P(\\theta)$", srt = 270)
text(x = corners[2]+.035, y = pretty(range(f_P_div_S_P(theta_vec)))[-c(1,6)], pretty(range(f_P_div_S_P(theta_vec)))[-c(1,6)], srt = 270)
mtext("$-\\chi$", side=3, line=1, at=q)
legend("topleft", legend=c("$|m(\\theta)|$", "$f_P(\\theta)$"), lty=c(1,4), col=c("black", "black"), bty="n", xpd=T)
par(mfrow=c(1,1))
dev.off()

pdf("./pdfs/prop_9_m_theta_f_P.pdf")
par(mfrow=c(1,1))
plot(theta_vec, abs(m1(theta_vec)), t="l", xlab=expression(theta), ylab=expression(abs(m(theta))), ylim=c(0,0.005))
abline(h=0, lty=3)
abline(v=-chi, lty=2)
points(theta_vec, f_P_div_S_P(theta_vec), t="l", lty=4)
par(mfrow=c(1,1))
dev.off()

print(c("cutoff chi", f_P_div_S_P(-chi)*2.5*sigma_theta^2 ))


## heteroskedasticity
n_const<-function(theta) 25 + theta*0

## increasing  and then constant at zero n
n_nonvec_piecewise<-function(theta) max(smallest_class_for_madeup_n, min(25.849,25.849+4.9655*theta))
n_piecewise<-function(theta) Vectorize(n_nonvec_piecewise)(theta)
smallest_class_for_madeup_n<-10
biggest_class_for_madeup_n<-40

phi_in_hetero<-function(n_in) function(theta)
{
  dnorm(theta, mean=q, sd=sqrt(sigma_bar_eps^2/n_in(theta)))
}

m2_hetero<-function(n_in) function(theta) m1(theta) * phi_in_hetero(n_in)(theta)

par(mfrow=c(2,1))
plot(theta_vec, m2_hetero(n_const)(theta_vec), t="l", col="black")
points(theta_vec, m2_hetero(n_piecewise)(theta_vec), t="l", col="red")
abline(v=-chi)

plot(theta_vec, log(abs(m2_hetero(n_const)(theta_vec))), t="l", col="black")
points(theta_vec, log(abs(m2_hetero(n_piecewise)(theta_vec))), t="l", col="red")
abline(v=-chi)
par(mfrow=c(1,1))

plot(theta_vec, abs(m2_hetero(n_const)(theta_vec))-abs(m2_hetero(n_piecewise)(theta_vec)), t="l", col="black")
