rm(list=ls())
library(tikzDevice)
## plotting parameters
one_pic_width<-4.5*(1.15^2)
one_pic_height<-one_pic_width
one_pic_pointsize<-12
two_pic_width<-4.5*1.25
two_pic_height<-two_pic_width
two_pic_pointsize<-12
three_pic_width<-4.5
three_pic_height<-three_pic_width
three_pic_pointsize<-12

library(truncnorm)

delta<-1
beta<-0
#sigma_theta<-.25
sigma_theta<-sqrt(0.011+0.035)
sigma_hetero<-0.25
kappa<--2*sigma_theta
c_EB<- kappa/2
sigma_eps_bar_homo<-0.195
theta_vec<-seq(-4*sigma_theta,0,length=100)

## expression \#2 \times \#3 in proposition 2
two_times_three<-function(gamma_sigma, kappa=kappa) function(gamma_c)
{
  k_norm<-kappa/sigma_theta
  k_mu_p_norm<-(kappa*(1-gamma_c))/(sigma_theta*sqrt(gamma_sigma))
  mu_p_norm<-(kappa*(-gamma_c))/(sigma_theta*sqrt(gamma_sigma))
  
  (1-pnorm(k_norm))/pnorm(k_norm) * pnorm(k_mu_p_norm)/(pnorm(mu_p_norm)-pnorm(k_mu_p_norm))
}

two_times_three_given_kappa<-function(gamma_sigma, gamma_c) function(kappa)
{
  k_norm<-kappa/sigma_theta
  k_mu_p_norm<-(kappa*(1-gamma_c))/(sigma_theta*sqrt(gamma_sigma))
  mu_p_norm<-(kappa*(-gamma_c))/(sigma_theta*sqrt(gamma_sigma))
  
  (1-pnorm(k_norm))/pnorm(k_norm) * pnorm(k_mu_p_norm)/(pnorm(mu_p_norm)-pnorm(k_mu_p_norm))
}

two_times_three_gamma_c_0<-function(x) (1-pnorm(x))/pnorm(x) * pnorm(x*sqrt(2))/(1/2-pnorm(x*sqrt(2)))
solve_kappa_gamma_c_0<-function(x) 2*pnorm(x*sqrt(2))-pnorm(x)
  
ratio<-function(x) dnorm(x*sqrt(2))/pnorm(x)
ratio_for_der<-function(x) dnorm(x)/pnorm(x)

## expression one in prop 2
one<-function(gamma_sigma, kappa=kappa) function(gamma_c)
{
  mu_p<-kappa*gamma_c
  sigma_P<-sigma_theta*sqrt(gamma_sigma)
  k_norm<-kappa/sigma_theta
  k_mu_p_norm<-(kappa*(1-gamma_c))/(sigma_theta*sqrt(gamma_sigma))
  mu_p_norm<-(kappa*(-gamma_c))/(sigma_theta*sqrt(gamma_sigma))
  
  upstairs<-mu_p + sigma_P*(-dnorm(k_mu_p_norm)/pnorm(k_mu_p_norm))
  downstairs<-mu_p + sigma_P*((dnorm(k_mu_p_norm)-dnorm(mu_p_norm))/(pnorm(mu_p_norm)-pnorm(k_mu_p_norm)))
  
  upstairs/downstairs
}

one_times_two_times_three<-function(gamma_sigma, kappa=kappa) function(gamma_c)
{
  one(gamma_sigma, kappa)(gamma_c) * two_times_three(gamma_sigma, kappa)(gamma_c)
}

one_times_two_times_three_given_kappa<-function(gamma_sigma, gamma_c) function(kappa)
{
  one_times_two_times_three(gamma_sigma, kappa)(gamma_c)  
}
kappa_tmp<-seq(-3*sigma_theta, 0, by=0.01)
par(mfrow=c(2,2))
plot(two_times_three(gamma_sigma=1/2, kappa=-1*sigma_theta))
plot(kappa_tmp, two_times_three_given_kappa(gamma_sigma=1/2, gamma_c=0.3)(kappa_tmp), t="l")
points(kappa_tmp, two_times_three_given_kappa(gamma_sigma=1/2, gamma_c=0.25)(kappa_tmp), t="l", lty=2)
abline(h=1)
plot(kappa_tmp, two_times_three_gamma_c_0(kappa_tmp), t="l")
abline(h=0)
plot(kappa_tmp, solve_kappa_gamma_c_0(kappa_tmp), t="l")
abline(h=1)
abline(v=-1)
par(mfrow=c(1,1))

phi_in_homo<-function(theta)
{
dnorm(theta, mean=c_EB/(delta+beta*theta), sd=sigma_eps_bar_homo)
}

phi_out<-function(theta)
{
dnorm(theta, mean=0, sd=sigma_theta)
}

phi_both_homo<-function(theta) phi_in_homo(theta) * phi_out(theta)

mu_both_homo<- ((c_EB/delta)*sigma_theta^2 + 0)/(sigma_eps_bar_homo^2 + sigma_theta^2)
sd_both_homo<-sqrt((sigma_theta^2*sigma_eps_bar_homo^2)/(sigma_eps_bar_homo^2+sigma_theta^2))
scale_both_homo<-dnorm(0,mean=-c_EB/delta, sd=sqrt(sigma_theta^2+sigma_eps_bar_homo^2))

mu_both_homo_given_gamma_tilde<-function(gamma_tilde_in) function(kappa_in)
{
c_EB_in<-kappa_in*gamma_tilde_in
 ((c_EB_in/delta)*sigma_theta^2 + 0)/(sigma_eps_bar_homo^2 + sigma_theta^2)
}
plot(theta_vec, phi_both_homo(theta_vec), t="l")
points(theta_vec, scale_both_homo*dnorm(theta_vec, mean=mu_both_homo, sd=sd_both_homo), col="red")

my_Deriv<-function(fun_in) function(x)
	{
	bump<-1e-3
	(fun_in(x+bump) - fun_in(x-bump))/(2*bump)
	}

plot(theta_vec, Vectorize(my_Deriv(dnorm))(theta_vec/2-1), t="l")
points(theta_vec, -(theta_vec/2-1)*dnorm(theta_vec/2-1), pch="x")

h<-function(theta) (c_EB*theta)/(delta + beta*theta)^2 * phi_in_homo(theta)
h_prime_num<-function(theta) Vectorize(my_Deriv(h))(theta)
h_prime_analytic<-function(theta) c_EB/delta^2 * phi_in_homo(theta) *(1+theta*(c_EB/delta-theta)/sigma_eps_bar_homo^2)

solve_max_h_analytical<-(c_EB/(delta*sigma_eps_bar_homo^2) - sqrt((c_EB/(delta*sigma_eps_bar_homo^2))^2+4/sigma_eps_bar_homo^2))/(2/sigma_eps_bar_homo^2)
approx_max_h<-c_EB/delta + delta*sigma_eps_bar_homo^2/c_EB
print(c(solve_max_h_analytical, approx_max_h))

## check binary approximation
binary_approx<-function(a,b) a*(1+b/(2*a^2))
thing_to_approx<-function(a,b) -sqrt(a^2+b)


plot(theta_vec, h(theta_vec), t="l")
points(theta_vec, h_prime_num(theta_vec), t="l", lty=3)
points(theta_vec, h_prime_analytic(theta_vec), t="l", pch="booyah", col="blue")
abline(a=0,b=0, lty=3)
abline(v=solve_max_h_analytical, lty=3)

par(mfrow=c(2,2))
plot(theta_vec, phi_in_homo(theta_vec), t="l")

plot(theta_vec, dtruncnorm(theta_vec, a=-Inf, b=kappa, mean=0, sd=sigma_theta), t="l", lty=2, main="conditional densities")
points(theta_vec, dtruncnorm(theta_vec, a=kappa, b=Inf, mean=0, sd=sigma_theta), t="l", lty=3)

plot(theta_vec, h(theta_vec), t="l", main="h(theta)")
abline(v=solve_max_h_analytical, lty=3, col="red")

plot(theta_vec[theta_vec<kappa], h(theta_vec[theta_vec<kappa]) * phi_out(theta_vec[theta_vec<kappa])/pnorm(kappa/sigma_theta), t="l", xlim=range(theta_vec))
points(theta_vec[theta_vec>=kappa], h(theta_vec[theta_vec>=kappa]) * phi_out(theta_vec[theta_vec>=kappa])/(1-pnorm(kappa/sigma_theta)), t="l")
abline(v=kappa, lty=3)
abline(v=-1)
par(mfrow=c(1,1))

# plot truncation correction bit
upstairs_ratio<-function(gamma_tilde) function(kappa_in) 
{
#	mu_p<-gamma_tilde*kappa_in/delta
mu_p<-mu_both_homo_given_gamma_tilde(gamma_tilde)(kappa_in)

	mu_p - sd_both_homo * dnorm((kappa_in-mu_p)/sd_both_homo)/pnorm((kappa_in-mu_p)/sd_both_homo)
}

downstairs_ratio<-function(gamma_tilde) function(kappa_in) 
{
#	mu_p<-gamma_tilde*kappa_in/delta
mu_p<-mu_both_homo_given_gamma_tilde(gamma_tilde)(kappa_in)

mu_p + sd_both_homo * (dnorm((kappa_in-mu_p)/sd_both_homo)-dnorm((0-mu_p)/sd_both_homo))/(pnorm((0-mu_p)/sd_both_homo)-pnorm((kappa_in-mu_p)/sd_both_homo))
}

ratio<-function(gamma_tilde) function(kappa_in) 
{
#mu_p<-gamma_tilde*kappa_in/delta
mu_p<-mu_both_homo_given_gamma_tilde(gamma_tilde)(kappa_in)

(pnorm((kappa_in-mu_p)/sd_both_homo) * upstairs_ratio(gamma_tilde)(kappa_in)) / ((pnorm((0-mu_p)/sd_both_homo)-pnorm((kappa_in-mu_p)/sd_both_homo)) * downstairs_ratio(gamma_tilde)(kappa_in)) 
}

ratio_with_trunc_prob_bits<-function(gamma_tilde) function(kappa_in) 
{
#mu_p<-gamma_tilde*kappa_in/delta
mu_p<-mu_both_homo_given_gamma_tilde(gamma_tilde)(kappa_in)

(pnorm((kappa_in-mu_p)/sd_both_homo) * upstairs_ratio(gamma_tilde)(kappa_in)) / ((pnorm((0-mu_p)/sd_both_homo)-pnorm((kappa_in-mu_p)/sd_both_homo)) * downstairs_ratio(gamma_tilde)(kappa_in))  * ((1-pnorm(kappa_in/sigma_theta))/(pnorm(kappa_in/sigma_theta)))
}

ratio_of_prob_bits<-function(gamma_tilde) function(kappa_in) 
{
#mu_p<-gamma_tilde*kappa_in/delta
mu_p<-mu_both_homo_given_gamma_tilde(gamma_tilde)(kappa_in)

(pnorm((kappa_in-mu_p)/sd_both_homo))/((pnorm((0-mu_p)/sd_both_homo)-pnorm((kappa_in-mu_p)/sd_both_homo)))  * ((1-pnorm(kappa_in/sigma_theta))/(pnorm(kappa_in/sigma_theta)))
}

kappa_vec<- seq(qnorm(0.01, mean=0, sd=sigma_theta), -1/100, length=50)

summary(((kappa_vec-mu_both_homo_given_gamma_tilde(0.25)(kappa_vec))/sd_both_homo)/(kappa_vec/sigma_theta))

plot(kappa_vec, log(ratio_of_prob_bits(.25)(kappa_vec)), t="l", col="black")
points(kappa_vec, log(ratio_of_prob_bits(.5)(kappa_vec)), t="l", col="red")
points(kappa_vec, log(ratio_of_prob_bits(.75)(kappa_vec)), t="l", col="blue")
abline(a=0,b=0, lty=3)

plot(kappa_vec, log(ratio_with_trunc_prob_bits(.25)(kappa_vec)), t="l", col="black")
points(kappa_vec, log(ratio_with_trunc_prob_bits(.5)(kappa_vec)), t="l", col="red")
points(kappa_vec, log(ratio_with_trunc_prob_bits(.75)(kappa_vec)), t="l", col="blue")


#### plots h(theta) with heteroskedastic errors
## for plotting h(theta) with heteroskedastic error
sigma_eps<- sqrt(0.030 + 0.923) # sigma_omega^2, sigma_epsilon^2 from Schochett and Chiang, Table B-2
sigma_sq_eps<-sigma_eps^2
smallest_class_for_madeup_n<-10
biggest_class_for_madeup_n<-40

## use like SD(theta)=sigma_sq_eps/n(theta)
### define class size functions at top
	## constant n
  n_const<-function(theta) 25 + theta*0

  ## increasing  and then constant at zero n
#  n_nonvec_piecewise<-function(theta) max(smallest_class_for_madeup_n, min(25.849,25.849+9.9225*theta))
  n_nonvec_piecewise<-function(theta) max(smallest_class_for_madeup_n, min(25.849,25.849+4.9655*theta))
  #n_nonvec_piecewise<-function(theta) max(smallest_class_for_madeup_n, min(25.01,25.01+0.01*theta))
  n_piecewise<-function(theta) Vectorize(n_nonvec_piecewise)(theta)

mean_n_const<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_const(x), lower=-Inf, upper=Inf)$value
mean_n_piecewise<-integrate(function(x) dnorm(x, mean=0, sd=sigma_theta)*n_piecewise(x), lower=-Inf, upper=Inf)$value

mean_n_const_given_lt_zero<-integrate(function(x) dtruncnorm(x, a=-Inf,b=0,mean=0, sd=sigma_theta)*n_const(x),  lower=-Inf, upper=0)$value
mean_n_piecewise_given_lt_zero<-integrate(function(x) dtruncnorm(x,a=-Inf,b=0, mean=0, sd=sigma_theta)*n_piecewise(x), lower=-Inf, upper=0)$value

mean_n_by_scenario<-c(const_mean=mean_n_const, const_conditional_mean=mean_n_const_given_lt_zero, piecewise_mean=mean_n_piecewise, piecewise_conditional_mean=mean_n_piecewise_given_lt_zero)
print(mean_n_by_scenario)


phi_in_hetero<-function(n_in) function(theta)
{
dnorm(theta, mean=c_EB/(delta+beta*theta), sd=sqrt(sigma_sq_eps/n_in(theta)))
}

h_hetero<-function(n_in) function(theta) (c_EB*theta)/(delta + beta*theta)^2 * phi_in_hetero(n_in)(theta)

### from sagemath; matches my derivative
delta_sigma<-sigma_eps_bar_homo       	

d_h_d_beta_sagemath<-function(theta) 
{
-1*(-1/2*sqrt(2)*c_EB*theta^2*exp(-1/2*(theta - c_EB/delta)^2/delta_sigma^2)/(sqrt(pi)*delta^2*delta_sigma^2) + 1/2*sqrt(2)*c_EB*(theta - c_EB/delta)^2*theta^2*exp(-1/2*(theta -c_EB/delta)^2/delta_sigma^2)/(sqrt(pi)*delta^2*delta_sigma^4))
}

d_h_d_beta_analytical<-function(theta)
{
1*(c_EB*theta/delta^2 * (theta/delta_sigma^2 - theta*(theta-c_EB/delta)^2/delta_sigma^4)*dnorm((c_EB/delta-theta)/delta_sigma, mean=0, sd=1))
}

d_h_d_beta_no_dnorm<-function(theta)
{
-1*(c_EB*theta/delta^2 * (-theta/delta_sigma^2 + theta*(theta-c_EB/delta)^2/delta_sigma^4))
}

par(mfrow=c(2,1))
plot(theta_vec, h_hetero(n_const)(theta_vec), t="l", col="black")
points(theta_vec, h_hetero(n_piecewise)(theta_vec), t="l", col="red")
plot(theta_vec, d_h_d_beta_analytical(theta_vec), t="l", col="blue")
abline(v=kappa)
par(mfrow=c(1,1))

## Figure 9
pdf("./pdfs/prop_2_heterosked_1.pdf")
plot(theta_vec, h_hetero(n_const)(theta_vec), t="l", col="black", main=expression(h(theta)), xlab = expression(theta), ylab="")
points(theta_vec, h_hetero(n_piecewise)(theta_vec), t="l", col="red")
legend("topleft", legend=c(expression(beta[sigma]==0), expression(beta[sigma]>0)), col=c("black", "red"), lty=c(rep(1,2)), bty="n")
abline(v=kappa, lty=3)
abline(h=0, lty=3)
dev.off()
pdf("./pdfs/prop_2_heterosked_2.pdf")
plot(theta_vec, d_h_d_beta_analytical(theta_vec), t="l", col="blue", main=expression(frac(d~h(theta), d~beta[sigma])), xlab =expression(theta), ylab="")
abline(v=kappa, lty=3)
abline(h=0, lty=3)
dev.off()

tikz("./pdfs/prop_2_heterosked_1.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
plot(theta_vec, h_hetero(n_const)(theta_vec), t="l", col="black", xlab = "Teacher quality $\\theta$", ylab="$h(\\theta)$")
points(theta_vec, h_hetero(n_piecewise)(theta_vec), t="l", col="red")
legend("topleft", legend=c("$\\beta_{\\sigma}=0$", "$\\beta_{\\sigma}>0$"), col=c("black", "red"), lty=c(rep(1,2)), bty="n")
abline(v=kappa, lty=3)
abline(h=0, lty=3)
dev.off()

tikz("./pdfs/prop_2_heterosked_2.tex", standAlone=F, width=two_pic_width, height=two_pic_height, pointsize=two_pic_pointsize)
plot(theta_vec, d_h_d_beta_analytical(theta_vec), t="l", col="blue", xlab ="Teacher quality $\\theta$", ylab="$\\frac{\\partial h(\\theta)}{\\partial \\beta_{\\sigma}}$")
abline(v=kappa, lty=3)
abline(h=0, lty=3)
dev.off()
